///////////////////////////////////////////////////////////////////////////////
// File		:	GLModel.h
// Creator	:	Ashley Bennett
// Date		:	Saturday 11 September 2005
// Purpose	:	A class to load and display MRF 3D models.
///////////////////////////////////////////////////////////////////////////////

#pragma once

//---INCLUDES---

#include <windows.h>
#include <gl/glut.h>
#include "Structures.h"


//---DEFINES---

//NONE


//---MODEL CLASS---

class GLModel
{
private:
	GLfloat			*m_vdata;			//Vertex Data: array of vertex points
	GLfloat			*m_ndata;			//Normal Data: array of normal vectors
	GLuint			*m_fdata;			//Face Data: array of indices to draw with
	GLfloat			*m_uvdata;			//UV Data: array of texture positions

	UINT			m_unNumVerts,		//Number of Verticies to load in
					m_unNumFaces,		//Number of faces to load in
					m_unNumTextures;	//Number of Textures to load in

	UINT			m_unTextureID;		//Which texture ID to use to draw this model's texture

	char			m_szTexture[_MAX_PATH];	//Path to the texture for this model

public:
	//Default Constructor
	GLModel();

	//Default Destructor
	~GLModel();

	///////////////////////////////////////////////////////////////////////////////
	// Function	:	Accessors
	// Creator	:	Ashley Bennett
	// Date		:	Sunday 12 September 2005
	// Purpose	:	Gets values from member variables.
	///////////////////////////////////////////////////////////////////////////////
	UINT		getNumVerts(void)		{ return m_unNumVerts; }
	UINT		getNumFaces(void)		{ return m_unNumFaces; }
	UINT		getNumTextures(void)	{ return m_unNumTextures; }
	UINT		getTextureID(void)		{ return m_unTextureID; }
	GLfloat*	getVertData(void)		{ return m_vdata; }
	GLfloat*	getNormalData(void)		{ return m_ndata; }
	GLuint*		getFaceData(void)		{ return m_fdata; }
	GLfloat*	getUVData(void)			{ return m_uvdata; }
	char*		getTexturePath(void)	{ return m_szTexture; }

	///////////////////////////////////////////////////////////////////////////////
	// Function	:	Modifiers
	// Creator	:	Ashley Bennett
	// Date		:	Sunday 12 September 2005
	// Purpose	:	Sets values in member variables.
	///////////////////////////////////////////////////////////////////////////////
	void setNumVerts(UINT unNumVerts)		{ m_unNumVerts = unNumVerts; }
	void setNumFaces(UINT unNumFaces)		{ m_unNumFaces = unNumFaces; }
	void setNumTextures(UINT unNumTextures)	{ m_unNumTextures = unNumTextures; }
	void setTextureID(UINT unTextureID)		{ m_unTextureID = unTextureID; }
	void setVertData(GLfloat* vdata)		{ m_vdata = vdata; }
	void setNormalData(GLfloat* ndata)		{ m_ndata = ndata; }
	void setFaceData(GLuint* fdata)			{ m_fdata = fdata; }
	void setUVData(GLfloat* uvdata)			{ m_uvdata = uvdata; }
	void setTexturePath(char* szPath)	{ strcpy(m_szTexture, szPath); }

	///////////////////////////////////////////////////////////////////////////////
	// Function	:	resetVertData
	// Creator	:	Ashley Bennett
	// Date		:	Sunday 12 September 2005
	// Purpose	:	Deletes any old data in the vertex data array, and resizes the
	// array to the length in m_unNumVerts.
	///////////////////////////////////////////////////////////////////////////////
	void resetVertData(void);

	///////////////////////////////////////////////////////////////////////////////
	// Function	:	resetNormalData
	// Creator	:	Ashley Bennett
	// Date		:	Sunday 12 September 2005
	// Purpose	:	Deletes any old data in the normal data array, and resizes the
	// array to the length in m_unNumVerts.
	///////////////////////////////////////////////////////////////////////////////
	void resetNormalData(void);

	///////////////////////////////////////////////////////////////////////////////
	// Function	:	resetFaceData
	// Creator	:	Ashley Bennett
	// Date		:	Sunday 12 September 2005
	// Purpose	:	Deletes any old data in the face data array, and resizes the
	// array to the length in m_unNumFaces.
	///////////////////////////////////////////////////////////////////////////////
	void resetFaceData(void);

	///////////////////////////////////////////////////////////////////////////////
	// Function	:	resetUVData
	// Creator	:	Ashley Bennett
	// Date		:	Sunday 12 September 2005
	// Purpose	:	Deletes any old data in the UV data array, and resizes the
	// array to the length in m_unNumVerts.
	///////////////////////////////////////////////////////////////////////////////
	void resetUVData(void);

	///////////////////////////////////////////////////////////////////////////////
	// Function	:	resetTextureData
	// Creator	:	Ashley Bennett
	// Date		:	Monday 19 September 2005
	// Purpose	:	Deletes clears the path to the model's texture and the number
	// of textures.
	///////////////////////////////////////////////////////////////////////////////
	void resetTextureData(void);

	///////////////////////////////////////////////////////////////////////////////
	// Function	:	loadTextures
	// Creator	:	Ashley Bennett
	// Date		:	Monday 19 September 2005
	// Purpose	:	Loads in the textures for this model and sets up necessary
	// variables, such as the Texture ID.
	///////////////////////////////////////////////////////////////////////////////
	void loadTextures(void);

	///////////////////////////////////////////////////////////////////////////////
	// Function	:	scale
	// Creator	:	Ashley Bennett
	// Date		:	Sunday 12 September 2005
	// Purpose	:	Scales the model by multiplying the verticies by the amount
	// passed into this function.
	///////////////////////////////////////////////////////////////////////////////
	void scale(float fScale);

	///////////////////////////////////////////////////////////////////////////////
	// Function	:	flipNormals
	// Creator	:	Ashley Bennett
	// Date		:	Sunday 12 September 2005
	// Purpose	:	Turns the normals to point the other way to make objects that
	// are "inside-out" correct again.
	///////////////////////////////////////////////////////////////////////////////
	void flipNormals(void);

	///////////////////////////////////////////////////////////////////////////////
	// Function	:	normalizeNormals
	// Creator	:	Ashley Bennett
	// Date		:	Sunday 12 September 2005
	// Purpose	:	Make all normals unit vectors to speed up processing.
	///////////////////////////////////////////////////////////////////////////////
	void normalizeNormals(void);

	///////////////////////////////////////////////////////////////////////////////
	// Function	:	loadXML
	// Creator	:	Ashley Bennett
	// Date		:	Sunday 12 September 2005
	// Purpose	:	Sets up a model by loading in data from an XML file.
	///////////////////////////////////////////////////////////////////////////////
	void loadXML(char *szPath);

	///////////////////////////////////////////////////////////////////////////////
	// Function	:	loadBIN
	// Creator	:	Ashley Bennett
	// Date		:	Sunday 12 September 2005
	// Purpose	:	Sets up a model by loading in data from an BIN file.
	///////////////////////////////////////////////////////////////////////////////
	void loadBIN(char *szPath);

	///////////////////////////////////////////////////////////////////////////////
	// Function	:	draw
	// Creator	:	Ashley Bennett
	// Date		:	Saturday 11 September 2005
	// Purpose	:	Draw a 3D model loaded from an MRF.
	///////////////////////////////////////////////////////////////////////////////
	void draw(void);
};
