///////////////////////////////////////////////////////////////////////////////
// File		:	GameExporterViewer.cpp
// Creator	:	Ashley Bennett
// Date		:	Thursday 15 September 2005
// Purpose	:	Defines the entry point for the application.
///////////////////////////////////////////////////////////////////////////////

//---INCLUDES---

#include "stdafx.h"
#include "GameExporterViewer.h"
#include "Game.h"


//---DEFINES---

#define MAX_LOADSTRING	100
#define WIN_WIDTH		800
#define WIN_HEIGHT		600


//---GLOBALS---

HINSTANCE	hInst;							// Current instance
TCHAR		szTitle[MAX_LOADSTRING];		// The title bar text
TCHAR		szWindowClass[MAX_LOADSTRING];	// The main window class name

//HWND		hWnd;							// Handle to the window //Now in Game.h for FPS counter


//---PROTOTYPES---

ATOM				MyRegisterClass(HINSTANCE hInstance);
BOOL				InitInstance(HINSTANCE, int);
LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	About(HWND, UINT, WPARAM, LPARAM);


//---FUNCTIONS---

int APIENTRY _tWinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPTSTR    lpCmdLine,
                     int       nCmdShow)
{
 	// TODO: Place code here.
	MSG msg;
	HACCEL hAccelTable;

	// Initialize global strings
	LoadString(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
	LoadString(hInstance, IDC_GAMEEXPORTERVIEWER, szWindowClass, MAX_LOADSTRING);
	MyRegisterClass(hInstance);

	// Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow)) 
	{
		return FALSE;
	}

	hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)IDC_GAMEEXPORTERVIEWER);

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) 
	{
		if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg)) 
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}

		//Reset the game timer every frame
		//g_Timer.Reset();
	}

	return (int) msg.wParam;
}



//
//  FUNCTION: MyRegisterClass()
//
//  PURPOSE: Registers the window class.
//
//  COMMENTS:
//
//    This function and its usage are only necessary if you want this code
//    to be compatible with Win32 systems prior to the 'RegisterClassEx'
//    function that was added to Windows 95. It is important to call this function
//    so that the application will get 'well formed' small icons associated
//    with it.
//
ATOM MyRegisterClass(HINSTANCE hInstance)
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX); 

	wcex.style			= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wcex.lpfnWndProc	= (WNDPROC)WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, (LPCTSTR)IDI_GAMEEXPORTERVIEWER);
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(0);
	wcex.lpszMenuName	= (LPCTSTR)IDC_GAMEEXPORTERVIEWER;
	wcex.lpszClassName	= szWindowClass;
	wcex.hIconSm		= LoadIcon(wcex.hInstance, (LPCTSTR)IDI_SMALL);

	return RegisterClassEx(&wcex);
}

//
//   FUNCTION: InitInstance(HANDLE, int)
//
//   PURPOSE: Saves instance handle and creates main window
//
//   COMMENTS:
//
//        In this function, we save the instance handle in a global variable and
//        create and display the main program window.
//
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
   hInst = hInstance; // Store instance handle in our global variable

   hWnd = CreateWindow(
	   szWindowClass,
	   szTitle,
	   WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
	   CW_USEDEFAULT,
	   0,
	   WIN_WIDTH,
	   WIN_HEIGHT,
	   NULL,
	   NULL,
	   hInstance,
	   NULL);

   if (!hWnd)
   {
      return FALSE;
   }

   ShowWindow(hWnd, nCmdShow);
   UpdateWindow(hWnd);

   return TRUE;
}

//
//  FUNCTION: WndProc(HWND, unsigned, WORD, LONG)
//
//  PURPOSE:  Processes messages for the main window.
//
//  WM_COMMAND	- process the application menu
//  WM_PAINT	- Paint the main window
//  WM_DESTROY	- post a quit message and return
//
//
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;
	static HDC hDC = NULL;			//GDI Device Context
	static HGLRC hRC = NULL;		//OpenGL rendering Context

	static int nTextureSelect	= ID_TEXTURES_BMP;	//Which menu items have a check mark by them
	static int nLightSelect		= ID_LIGHT_ON;
	static int nRenderSelect	= ID_RENDERMODE_SOLID;
	static int nCullingSelect	= ID_BACKFACECULLING_ON;
	static int nWindingSelect	= ID_WINDINGORDER_COUNTERCLOCKWISE;


	switch (message) 
	{
	case WM_INITMENUPOPUP:
		{
			//When the menus are opening, check mark the options
			HMENU hMenu = HMENU(wParam);
			CheckMenuItem(hMenu, nTextureSelect, MF_CHECKED);
			CheckMenuItem(hMenu, nLightSelect, MF_CHECKED);
			CheckMenuItem(hMenu, nRenderSelect, MF_CHECKED);
			CheckMenuItem(hMenu, nCullingSelect, MF_CHECKED);
			CheckMenuItem(hMenu, nWindingSelect, MF_CHECKED);
		}
		break;
	case WM_COMMAND:
		{
			wmId    = LOWORD(wParam);
			wmEvent = HIWORD(wParam);
			HMENU hMenu = GetMenu(hWnd);

			// Parse the menu selections:
			switch (wmId)
			{
			case ID_TEXTURES_BMP:
				{
					CheckMenuItem(hMenu, nTextureSelect, MF_UNCHECKED);
					nTextureSelect = ID_TEXTURES_BMP;

					glEnable(GL_TEXTURE_2D);
					g_unTextureMode = TEX_BMP;
					LoadTextures();
				}
				break;
			case ID_TEXTURES_BMPCOMPRESSED:
				{
					CheckMenuItem(hMenu, nTextureSelect, MF_UNCHECKED);
					nTextureSelect = ID_TEXTURES_BMPCOMPRESSED;

					glEnable(GL_TEXTURE_2D);
					g_unTextureMode = TEX_COMPRESSEDBMP;
					LoadTextures();
				}
				break;
			case ID_TEXTURES_BMPCUBEMAP:
				{
					CheckMenuItem(hMenu, nTextureSelect, MF_UNCHECKED);
					nTextureSelect = ID_TEXTURES_BMPCUBEMAP;

					glEnable(GL_TEXTURE_2D);
					g_unTextureMode = TEX_CUBEMAPBMP;
					LoadTextures();
				}
				break;
			case ID_TEXTURES_TGA:
				{
					CheckMenuItem(hMenu, nTextureSelect, MF_UNCHECKED);
					nTextureSelect = ID_TEXTURES_TGA;

					glEnable(GL_TEXTURE_2D);
					g_unTextureMode = TEX_TGA;
					LoadTextures();
				}
				break;
			case ID_TEXTURES_TGACOMPRESED:
				{
					CheckMenuItem(hMenu, nTextureSelect, MF_UNCHECKED);
					nTextureSelect = ID_TEXTURES_TGACOMPRESED;

					glEnable(GL_TEXTURE_2D);
					g_unTextureMode = TEX_COMPRESSEDTGA;
					LoadTextures();
				}
				break;
			case ID_TEXTURES_DDS:
				{
					CheckMenuItem(hMenu, nTextureSelect, MF_UNCHECKED);
					nTextureSelect = ID_TEXTURES_DDS;

					glEnable(GL_TEXTURE_2D);
					g_unTextureMode = TEX_DDS;
					LoadTextures();
				}
				break;
			case ID_TEXTURES_DDSCUBEMAP:
				{
					CheckMenuItem(hMenu, nTextureSelect, MF_UNCHECKED);
					nTextureSelect = ID_TEXTURES_DDSCUBEMAP;

					glEnable(GL_TEXTURE_2D);
					g_unTextureMode = TEX_CUBEMAPDDS;
					LoadTextures();
				}
				break;
			case ID_TEXTURES_MODEL:
				{
					CheckMenuItem(hMenu, nTextureSelect, MF_UNCHECKED);
					nTextureSelect = ID_TEXTURES_MODEL;

					glEnable(GL_TEXTURE_2D);
					g_unTextureMode = TEX_MODEL;
				}
				break;
			case ID_TEXTURES_OFF:
				{
					CheckMenuItem(hMenu, nTextureSelect, MF_UNCHECKED);
					nTextureSelect = ID_TEXTURES_OFF;

					glDisable(GL_TEXTURE_2D);
					g_unTextureMode = TEX_NONE;
				}
				break;

			case ID_LIGHT_ON:
				{
					CheckMenuItem(hMenu, nLightSelect, MF_UNCHECKED);
					nLightSelect = ID_LIGHT_ON;

					glEnable(GL_LIGHTING);
				}
				break;
			case ID_LIGHT_OFF:
				{
					CheckMenuItem(hMenu, nLightSelect, MF_UNCHECKED);
					nLightSelect = ID_LIGHT_OFF;

					glDisable(GL_LIGHTING);
				}
				break;

			case ID_RENDERMODE_SOLID:
				{
					CheckMenuItem(hMenu, nRenderSelect, MF_UNCHECKED);
					nRenderSelect = ID_RENDERMODE_SOLID;

					glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
				}
				break;
			case ID_RENDERMODE_WIREFRAME:
				{
					CheckMenuItem(hMenu, nRenderSelect, MF_UNCHECKED);
					nRenderSelect = ID_RENDERMODE_WIREFRAME;

					glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
				}
				break;

			case ID_BACKFACECULLING_ON:
				{
					CheckMenuItem(hMenu, nCullingSelect, MF_UNCHECKED);
					nCullingSelect = ID_BACKFACECULLING_ON;

					glEnable(GL_CULL_FACE);
				}
				break;
			case ID_BACKFACECULLING_OFF:
				{
					CheckMenuItem(hMenu, nCullingSelect, MF_UNCHECKED);
					nCullingSelect = ID_BACKFACECULLING_OFF;

					glDisable(GL_CULL_FACE);
				}
				break;

			case ID_WINDINGORDER_CLOCKWISE:
				{
					CheckMenuItem(hMenu, nWindingSelect, MF_UNCHECKED);
					nWindingSelect = ID_WINDINGORDER_CLOCKWISE;

					glFrontFace(GL_CW);
				}
				break;
			case ID_WINDINGORDER_COUNTERCLOCKWISE:
				{
					CheckMenuItem(hMenu, nWindingSelect, MF_UNCHECKED);
					nWindingSelect = ID_WINDINGORDER_COUNTERCLOCKWISE;

					glFrontFace(GL_CW);
				}
				break;

			case ID_MODEL_FILPNORMALS:
				g_Model.flipNormals();
				break;
			case ID_MODEL_SCALEDOWN10:
				g_Model.scale(0.9f);
				break;
			case ID_MODEL_SCALEUP10:
				g_Model.scale(1.1f);
				break;

			case ID_OPTIONS_RESETCAMERA:
				ResetWorld();
				break;

			case ID_FILE_OPEN:
				{
					OPENFILENAME ofn;       // common dialog box structure
					char szFile[260];       // buffer for file name

					// Initialize OPENFILENAME
					ZeroMemory(&ofn, sizeof(ofn));
					ofn.lStructSize = sizeof(ofn);
					ofn.hwndOwner = hWnd;
					ofn.lpstrFile = szFile;
					//
					// Set lpstrFile[0] to '\0' so that GetOpenFileName does not 
					// use the contents of szFile to initialize itself.
					//
					ofn.lpstrFile[0] = '\0';
					ofn.nMaxFile = sizeof(szFile);
					ofn.lpstrFilter = "XML\0*.XML\0BIN\0*.BIN\0All\0*.*\0";
					ofn.nFilterIndex = 1;
					ofn.lpstrFileTitle = NULL;
					ofn.nMaxFileTitle = 0;
					ofn.lpstrInitialDir = NULL;
					ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

					// Display the Open dialog box. 

					if (GetOpenFileName(&ofn) == TRUE)
					{
						// Use the correct function to open the file based on extension
						if (!strcmp(&ofn.lpstrFile[ofn.nFileExtension], "xml"))
							g_Model.loadXML(ofn.lpstrFile);
						else if (!strcmp(&ofn.lpstrFile[ofn.nFileExtension], "bin"))
							g_Model.loadBIN(ofn.lpstrFile);

						//Not a recognised format, so give them the option of opening anyway
						else if (MessageBox(hWnd, "Extension not recognised. Open as BIN?", "File Type Error", MB_YESNO) == IDYES)
							g_Model.loadBIN(ofn.lpstrFile);

						//Either way, reset the world and camera
						ResetWorld();

						//If the model has textures, lets show them
						if (g_Model.getNumTextures())
						{
							CheckMenuItem(hMenu, nTextureSelect, MF_UNCHECKED);
							nTextureSelect = ID_TEXTURES_MODEL;

							g_unTextureMode = TEX_MODEL;
						}
						else
						{
							CheckMenuItem(hMenu, nTextureSelect, MF_UNCHECKED);
							nTextureSelect = ID_TEXTURES_BMP;

							g_unTextureMode = TEX_BMP;
						}
					}
				}
				break;
			case IDM_ABOUT:
				DialogBox(hInst, (LPCTSTR)IDD_ABOUTBOX, hWnd, (DLGPROC)About);
				break;
			case IDM_EXIT:
				DestroyWindow(hWnd);
				break;
			default:
				return DefWindowProc(hWnd, message, wParam, lParam);
			}

			// Force a repaint
			InvalidateRect(hWnd, NULL, true);
		}
		break;

	case WM_KEYDOWN:
		{
			//float fDeltaTime = g_Timer.GetElapsedSeconds();
			//float fDeltaTime = g_Timer.GetTime();
			float fDeltaTime = 1.0f;

			switch(wParam)
			{
			case VK_ESCAPE:
				//Quit
				PostMessage(hWnd, WM_DESTROY, 0, 0);
				break;

			case VK_UP:
				{
					//Move the camera
					g_frCamera.MoveForward(CAM_MOVE * fDeltaTime);
					//Move the frustum with the camera
					g_Frustum.Transform(g_frCamera);
				}
				break;
			case VK_DOWN:
				{
					//Move the camera
					g_frCamera.MoveForward(-CAM_MOVE * fDeltaTime);
					//Move the frustum with the camera
					g_Frustum.Transform(g_frCamera);
				}
				break;

			case VK_LEFT:
			case 'A':
				{
					//Move the camera
					g_frCamera.MoveRight(-CAM_MOVE * fDeltaTime);
					//Move the frustum with the camera
					g_Frustum.Transform(g_frCamera);
				}
				break;
			case VK_RIGHT:
			case 'D':
				{
					//Move the camera
					g_frCamera.MoveRight(CAM_MOVE * fDeltaTime);
					//Move the frustum with the camera
					g_Frustum.Transform(g_frCamera);
				}
				break;

			case 'W':
				{
					//Move the camera
					g_frCamera.MoveUp(-CAM_MOVE * fDeltaTime);
					//Move the frustum with the camera
					g_Frustum.Transform(g_frCamera);
				}
				break;
			case 'S':
				{
					//Move the camera
					g_frCamera.MoveUp(CAM_MOVE * fDeltaTime);
					//Move the frustum with the camera
					g_Frustum.Transform(g_frCamera);
				}
				break;
			}

			//Reset the game timer every keypress
			//g_Timer.Reset();

			// Force a repaint
			InvalidateRect(hWnd, NULL, true);
		}
		break;

	case WM_LBUTTONDOWN:
		{
			//If you don't set the capture you won't get mouse move
			//messages when the mouse moves outside the window.
			SetCapture(hWnd);

			//Get the mouse point
			g_ptOldMouse.x = LOWORD(lParam);
			g_ptOldMouse.y = HIWORD(lParam);

			//Save the state to know how to process mouse movement
			g_unMouseState = M_ROTATE;
		}
		break;
	case WM_RBUTTONDOWN:
		{
			//If you don't set the capture you won't get mouse move
			//messages when the mouse moves outside the window.
			SetCapture(hWnd);

			//Get the mouse point
			g_ptOldMouse.x = LOWORD(lParam);
			g_ptOldMouse.y = HIWORD(lParam);

			//Save the state to know how to process mouse movement
			g_unMouseState = M_TRANSLATE;
		}
		break;
	case WM_MBUTTONDOWN:
		{
			//If you don't set the capture you won't get mouse move
			//messages when the mouse moves outside the window.
			SetCapture(hWnd);

			//Get the mouse point
			g_ptOldMouse.x = LOWORD(lParam);
			g_ptOldMouse.y = HIWORD(lParam);

			//Save the state to know how to process mouse movement
			g_unMouseState = M_ZOOM;
		}
		break;

	case WM_LBUTTONUP:
	case WM_RBUTTONUP:
	case WM_MBUTTONUP:
		{
			//Release the mouse from capture now we are finished.
			ReleaseCapture();
			g_unMouseState = M_NONE;
		}
		break;

	case WM_MOUSEMOVE:
		{
			//If we have a mouse button down
			if (g_unMouseState) {
				//Get the mouse point
				g_ptMouse.x = LOWORD(lParam);
				g_ptMouse.y = HIWORD(lParam);

				/* Win32 is pretty braindead about the x, y position that
				it returns when the mouse is off the left or top edge
				of the window (due to them being unsigned). therefore,
				roll the Win32's 0..2^16 pointer co-ord range to the
				more amenable (and useful) 0..+/-2^15. */
				if(g_ptMouse.x & (1 << 15)) g_ptMouse.x -= (1 << 16);
				if(g_ptMouse.y & (1 << 15)) g_ptMouse.y -= (1 << 16);
				WorldUpdate();

				//Save the mouse point
				g_ptOldMouse.x = g_ptMouse.x;
				g_ptOldMouse.y = g_ptMouse.y;

				// Force a repaint
				InvalidateRect(hWnd, NULL, TRUE);
			}
		}
		break;

	case WM_SIZE:
		{
			int nWidth	= LOWORD(lParam);
			int nHeight	= HIWORD(lParam);

			ChangeSize(nWidth, nHeight);

			// Force a repaint
			InvalidateRect(hWnd, NULL, true);
		}
		break;
	case WM_PAINT:
		{
			//GetInput();

			Render();

			SwapBuffers(hDC);

			ValidateRect(hWnd, 0);	//Stop rendering
		}
		break;
	case WM_CREATE:
		{
			//Save the hDC once into our static variable
			hDC = GetDC(hWnd);

			PIXELFORMATDESCRIPTOR pfd = {
				sizeof(PIXELFORMATDESCRIPTOR),
				1,							//Version number (Always 1)
				PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER,
				PFD_TYPE_RGBA,
				32,							//Color Buffer Depth (Always just put 32)
				0,0,0,0,0,0,0,0,0,0,0,0,0,	//13 0's
				16,							//Depth of Z-Buffer (Usually 16 or 24)
				8,							//Depth of Stencil Buffer
				0,0,0,0,0,0};				//7 0's

			//See what the Graphics card gave us
			int pixelFormat = ChoosePixelFormat(hDC, &pfd);

			//Sometimes more stuff goes here

			//See what the Graphics card gave us
			SetPixelFormat(hDC, pixelFormat, &pfd);

			//Create the RC
			hRC = wglCreateContext(hDC);

			//Bind OpenGL to this window
			wglMakeCurrent(hDC, hRC);

			//Save the current working directory (where the application is)
			_getcwd( g_szAppDir, _MAX_PATH );

			//Initialize OpenGL
			SetupGL();

			//Setup variables for the game
			GameStartup();

			//If the model has textures, set the correct menu check
			if (g_Model.getNumTextures())
				nTextureSelect = ID_TEXTURES_MODEL;
			else
				g_unTextureMode = TEX_BMP;
		}
		break;
	case WM_DESTROY:
		{
			//Cleanup OpenGL
			ShutdownGL();

			//Unbind OpenGL from this window
			wglMakeCurrent(hDC, NULL);

			//Destroy the RC
			wglDeleteContext(hRC);

			ReleaseDC(hWnd, hDC);
			PostQuitMessage(0);
		}
		break;
	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return 0;
}

// Message handler for about box.
LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
		return TRUE;

	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) 
		{
			EndDialog(hDlg, LOWORD(wParam));
			return TRUE;
		}
		break;
	}
	return FALSE;
}
