///////////////////////////////////////////////////////////////////////////////
// File		:	MyXML.h
// Creator	:	Ashley Bennett
// Date		:	Saturday 10 September 2005
// Purpose	:	Contains all functionality for an XML parser that loads in MRF
// Files, (thats 3D models) into an OpenGL Format.
///////////////////////////////////////////////////////////////////////////////

#pragma once

//---INCLUDES---

#include <msxml2.h>
#include <fstream>
using namespace std;

#include "GLModel.h"


//---DEFINES---

//TYPES OF FILES WE COULD LOAD
enum { NONE_FILE = 0, MRF_FILE };

//FLAGS TO MARK WHERE WE ARE IN THE FILES
enum { NOWHERE = 0 };																			//COMMON
enum { M_MESH = 1, M_NAME, M_PATH, M_NUM_VERTS, M_NUM_FACES, M_NUM_TEXTURES,					//MRF
		M_LIST_TEXTURES, M_LIST_TEXTURES_TEXTURE,
		M_LIST_VERTS, M_LIST_VERTS_VERT, M_LIST_VERTS_VERT_X, M_LIST_VERTS_VERT_Y, M_LIST_VERTS_VERT_Z,
		M_LIST_VERTS_VERT_NORMX, M_LIST_VERTS_VERT_NORMY, M_LIST_VERTS_VERT_NORMZ,
		M_LIST_VERTS_VERT_U, M_LIST_VERTS_VERT_V,
		M_LIST_FACES, M_LIST_FACES_FACE, M_LIST_FACES_FACE_A, M_LIST_FACES_FACE_B, M_LIST_FACES_FACE_C, };


//---MYXML---

class MyXML : public ISAXContentHandler
{
private:
	//---STRUCTURES---

	//NONE


	//---MY MEMBERS---

	//General XML parsing variables
	UINT			m_unType,			//Type of file we have open (enum value)
					m_unPos,			//Position in XML file (enum value)
					m_unListCount;		//Where we are in a List (for array access)

	//Variables from MRF Files
	GLModel			*m_pglModel;		//A model to load the data into

public:
	//---MY METHODS---

	//default constructor
	MyXML(void)
	{
		Reset();
	}

	//default destructor
	virtual ~MyXML(void) {}


	/////////////////////////////////////////////////////////////////////////
	//	Function		:	Reset
	//
	//	Last Modified	:	July 25, 2005
	//
	//	Purpose			:	Clear all the member variables in this parser.
	/////////////////////////////////////////////////////////////////////////
	void Reset(void)
	{
		m_unType = NONE_FILE;
		m_unPos = 0;

		//For MRF File Variables
		m_pglModel = 0;
	}


	/////////////////////////////////////////////////////////////////////////
	//	Function		:	Accessors
	//
	//	Last Modified	:	July 25, 2005
	//
	//	Purpose			:	For getting the values of MyXML member variables.
	/////////////////////////////////////////////////////////////////////////

	//For MRF File Variables
	GLModel* getGLModel(void) { return m_pglModel; }


	/////////////////////////////////////////////////////////////////////////
	//	Function		:	Modifiers
	//
	//	Last Modified	:	July 25, 2005
	//
	//	Purpose			:	For setting the values of MyXML member variables.
	/////////////////////////////////////////////////////////////////////////

	//For MRF File Variables
	void setGLModel(GLModel *pglModel) { m_pglModel = pglModel; }


public:
	/////////////////////////////////////////////////////////////////////////
	//	Function		:	Overrides
	//
	//	Last Modified	:	July 20, 2005
	//
	//	Purpose			:
	//	Need to overload these to get ISAXContentHandler working.
	/////////////////////////////////////////////////////////////////////////

	virtual HRESULT STDMETHODCALLTYPE putDocumentLocator( 
		/* [in] */ ISAXLocator *pLocator)
	{ return S_OK; }

	virtual HRESULT STDMETHODCALLTYPE startDocument( void)
	{ return S_OK; }

	virtual HRESULT STDMETHODCALLTYPE endDocument( void)
	{ return S_OK; }

	virtual HRESULT STDMETHODCALLTYPE startPrefixMapping( 
		/* [in] */ const wchar_t *pwchPrefix,
		/* [in] */ int cchPrefix,
		/* [in] */ const wchar_t *pwchUri,
		/* [in] */ int cchUri)
	{ return S_OK; }

	virtual HRESULT STDMETHODCALLTYPE endPrefixMapping( 
		/* [in] */ const wchar_t *pwchPrefix,
		/* [in] */ int cchPrefix)
	{ return S_OK; }

	virtual HRESULT STDMETHODCALLTYPE ignorableWhitespace( 
		/* [in] */ const wchar_t *pwchChars,
		/* [in] */ int cchChars)
	{ return S_OK; }

	virtual HRESULT STDMETHODCALLTYPE processingInstruction( 
		/* [in] */ const wchar_t *pwchTarget,
		/* [in] */ int cchTarget,
		/* [in] */ const wchar_t *pwchData,
		/* [in] */ int cchData)
	{ return S_OK; }

	virtual HRESULT STDMETHODCALLTYPE skippedEntity( 
		/* [in] */ const wchar_t *pwchName,
		/* [in] */ int cchName)
	{ return S_OK; }


	/////////////////////////////////////////////////////////////////////////
	//	Function		:	COM Essentials
	//
	//	Last Modified	:	July 20, 2005
	//
	//	Purpose			:
	//	For starting and ending COM objects, such as ISAXContentHandler.
	/////////////////////////////////////////////////////////////////////////
	unsigned long __stdcall AddRef(void) { return 0; }
	unsigned long __stdcall Release(void) { return 0; }
	long __stdcall QueryInterface(const struct _GUID &,void ** ) { return 0; }


	/////////////////////////////////////////////////////////////////////////
	//	Function		:	startElement
	//
	//	Last Modified	:	July 25, 2005
	//
	//	Purpose			:
	//	Keep track of where we are in the file when entering an XML tag.
	//	Will also allocate memory for the m_TileMap array on entering the
	//	<tilelist> tag.
	/////////////////////////////////////////////////////////////////////////
	virtual HRESULT STDMETHODCALLTYPE startElement( 
		/* [in] */ const wchar_t *pwchNamespaceUri,
		/* [in] */ int cchNamespaceUri,
		/* [in] */ const wchar_t *pwchLocalName,
		/* [in] */ int cchLocalName,
		/* [in] */ const wchar_t *pwchQName,
		/* [in] */ int cchQName,
		/* [in] */ ISAXAttributes *pAttributes)
	{
		char tagName[128];
		memset(tagName, 0, 128);
		wcstombs(tagName, pwchLocalName, cchLocalName);

		//common header

		//NONE

		//MRF file
		if (!strcmp(tagName, "mesh"))
		{
			m_unPos = M_MESH;
			m_unType = MRF_FILE;
		}
		else if (m_unType == MRF_FILE)
		{
			if (!strcmp(tagName, "name"))
			{
				m_unPos = M_NAME;
			}
			else if (!strcmp(tagName, "path"))
			{
				m_unPos = M_PATH;
			}
			else if (!strcmp(tagName, "numVerts"))
			{
				m_unPos = M_NUM_VERTS;
			}
			else if (!strcmp(tagName, "numFaces"))
			{
				m_unPos = M_NUM_FACES;
			}
			else if (!strcmp(tagName, "numTextures"))
			{
				m_unPos = M_NUM_TEXTURES;
			}

			else if (!strcmp(tagName, "textureList"))
			{
				m_unPos = M_LIST_TEXTURES;
				m_unListCount = 0;
			}
			else if (!strcmp(tagName, "texture"))
			{
				m_unPos = M_LIST_TEXTURES_TEXTURE;
			}

			else if (!strcmp(tagName, "vertList"))
			{
				m_unPos = M_LIST_VERTS;
				m_unListCount = 0;
			}
			else if (!strcmp(tagName, "vertex"))
			{
				m_unPos = M_LIST_VERTS_VERT;
			}
			else if (!strcmp(tagName, "vertX"))
			{
				m_unPos = M_LIST_VERTS_VERT_X;
			}
			else if (!strcmp(tagName, "vertY"))
			{
				m_unPos = M_LIST_VERTS_VERT_Y;
			}
			else if (!strcmp(tagName, "vertZ"))
			{
				m_unPos = M_LIST_VERTS_VERT_Z;
			}
			else if (!strcmp(tagName, "vertNX"))
			{
				m_unPos = M_LIST_VERTS_VERT_NORMX;
			}
			else if (!strcmp(tagName, "vertNY"))
			{
				m_unPos = M_LIST_VERTS_VERT_NORMY;
			}
			else if (!strcmp(tagName, "vertNZ"))
			{
				m_unPos = M_LIST_VERTS_VERT_NORMZ;
			}
			else if (!strcmp(tagName, "vertU"))
			{
				m_unPos = M_LIST_VERTS_VERT_U;
			}
			else if (!strcmp(tagName, "vertV"))
			{
				m_unPos = M_LIST_VERTS_VERT_V;
			}

			else if (!strcmp(tagName, "faceList"))
			{
				m_unPos = M_LIST_FACES;
				m_unListCount = 0;
			}
			else if (!strcmp(tagName, "face"))
			{
				m_unPos = M_LIST_FACES_FACE;
			}
			else if (!strcmp(tagName, "faceA"))
			{
				m_unPos = M_LIST_FACES_FACE_A;
			}
			else if (!strcmp(tagName, "faceB"))
			{
				m_unPos = M_LIST_FACES_FACE_B;
			}
			else if (!strcmp(tagName, "faceC"))
			{
				m_unPos = M_LIST_FACES_FACE_C;
			}
		}

		return S_OK;
	}

	/////////////////////////////////////////////////////////////////////////
	//	Function		:	endElement
	//
	//	Last Modified	:	July 25, 2005
	//
	//	Purpose			:
	//	Keep track of where we are in the file when leaving an XML tag.
	//	On leaving <tile>, we'll move to the next array index in m_TileMap.
	/////////////////////////////////////////////////////////////////////////
	virtual HRESULT STDMETHODCALLTYPE endElement( 
		/* [in] */ const wchar_t *pwchNamespaceUri,
		/* [in] */ int cchNamespaceUri,
		/* [in] */ const wchar_t *pwchLocalName,
		/* [in] */ int cchLocalName,
		/* [in] */ const wchar_t *pwchQName,
		/* [in] */ int cchQName)
	{
		char tagName[128];
		memset(tagName, 0, 128);
		wcstombs(tagName, pwchLocalName, cchLocalName);

		//MRF file
		if (m_unType == MRF_FILE)
		{
			if (!strcmp(tagName, "mesh"))
			{
				m_unPos = NOWHERE;
				m_unType = NONE_FILE;
			}
			else if (!strcmp(tagName, "name"))
			{
				m_unPos = M_MESH;
			}
			else if (!strcmp(tagName, "path"))
			{
				m_unPos = M_MESH;
			}
			else if (!strcmp(tagName, "numVerts"))
			{
				m_unPos = M_MESH;
			}
			else if (!strcmp(tagName, "numFaces"))
			{
				m_unPos = M_MESH;
			}
			else if (!strcmp(tagName, "numTextures"))
			{
				m_unPos = M_MESH;
			}

			else if (!strcmp(tagName, "textureList"))
			{
				m_unPos = M_MESH;
			}
			else if (!strcmp(tagName, "texture"))
			{
				m_unPos = M_LIST_TEXTURES;
				m_unListCount++;
			}

			else if (!strcmp(tagName, "vertList"))
			{
				m_unPos = M_MESH;
			}
			else if (!strcmp(tagName, "vertex"))
			{
				m_unPos = M_LIST_VERTS;
				m_unListCount++;
			}
			else if (!strcmp(tagName, "vertX"))
			{
				m_unPos = M_LIST_VERTS_VERT;
			}
			else if (!strcmp(tagName, "vertY"))
			{
				m_unPos = M_LIST_VERTS_VERT;
			}
			else if (!strcmp(tagName, "vertZ"))
			{
				m_unPos = M_LIST_VERTS_VERT;
			}
			else if (!strcmp(tagName, "vertNX"))
			{
				m_unPos = M_LIST_VERTS_VERT;
			}
			else if (!strcmp(tagName, "vertNY"))
			{
				m_unPos = M_LIST_VERTS_VERT;
			}
			else if (!strcmp(tagName, "vertNZ"))
			{
				m_unPos = M_LIST_VERTS_VERT;
			}
			else if (!strcmp(tagName, "vertU"))
			{
				m_unPos = M_LIST_VERTS_VERT;
			}
			else if (!strcmp(tagName, "vertV"))
			{
				m_unPos = M_LIST_VERTS_VERT;
			}

			else if (!strcmp(tagName, "faceList"))
			{
				m_unPos = M_MESH;
			}
			else if (!strcmp(tagName, "face"))
			{
				m_unPos = M_LIST_FACES;
				m_unListCount++;
			}
			else if (!strcmp(tagName, "faceA"))
			{
				m_unPos = M_LIST_FACES_FACE;
			}
			else if (!strcmp(tagName, "faceB"))
			{
				m_unPos = M_LIST_FACES_FACE;
			}
			else if (!strcmp(tagName, "faceC"))
			{
				m_unPos = M_LIST_FACES_FACE;
			}
		}

		return S_OK;
	}

	/////////////////////////////////////////////////////////////////////////
	//	Function		:	characters
	//
	//	Last Modified	:	July 25, 2005
	//
	//	Purpose			:
	//	Actually read in the data in the tag and save it somewhere useful.
	/////////////////////////////////////////////////////////////////////////
	virtual HRESULT STDMETHODCALLTYPE characters( 
		/* [in] */ const wchar_t *pwchChars,
		/* [in] */ int cchChars)
	{
		char content[256];
		memset(content, 0, 256);
		wcstombs(content, pwchChars, cchChars);

		//MRF file
		if (m_unType == MRF_FILE)
		{
			if (m_unPos == M_NAME)
			{
			}
			else if (m_unPos == M_PATH)
			{
			}
			else if (m_unPos == M_NUM_VERTS)
			{
				m_pglModel->setNumVerts( atoi(content) );
				m_pglModel->resetVertData();
				m_pglModel->resetNormalData();
				m_pglModel->resetUVData();
			}
			else if (m_unPos == M_NUM_FACES)
			{
				m_pglModel->setNumFaces( atoi(content) );
				m_pglModel->resetFaceData();
			}
			else if (m_unPos == M_NUM_TEXTURES)
			{
				m_pglModel->setNumTextures( atoi(content) );
			}

			else if (m_unPos == M_LIST_TEXTURES_TEXTURE)
			{
				m_pglModel->setTexturePath(content);
				m_pglModel->loadTextures();
			}

			else if (m_unPos == M_LIST_VERTS_VERT_X)
			{
				//m_pglModel->getVertData()[m_unListCount].data[0] = atof(content);
				m_pglModel->getVertData()[m_unListCount * 3] = atof(content);
			}
			else if (m_unPos == M_LIST_VERTS_VERT_Y)
			{
				//m_pglModel->getVertData()[m_unListCount].data[1] = atof(content);
				m_pglModel->getVertData()[m_unListCount * 3 + 1] = atof(content);
			}
			else if (m_unPos == M_LIST_VERTS_VERT_Z)
			{
				//m_pglModel->getVertData()[m_unListCount].data[2] = atof(content);
				m_pglModel->getVertData()[m_unListCount * 3 + 2] = atof(content);
			}

			else if (m_unPos == M_LIST_VERTS_VERT_NORMX)
			{
				//m_pglModel->getNormalData()[m_unListCount].data[0] = atof(content);
				m_pglModel->getNormalData()[m_unListCount * 3] = atof(content);
			}
			else if (m_unPos == M_LIST_VERTS_VERT_NORMY)
			{
				//m_pglModel->getNormalData()[m_unListCount].data[1] = atof(content);
				m_pglModel->getNormalData()[m_unListCount * 3 + 1] = atof(content);
			}
			else if (m_unPos == M_LIST_VERTS_VERT_NORMZ)
			{
				//m_pglModel->getNormalData()[m_unListCount].data[2] = atof(content);
				m_pglModel->getNormalData()[m_unListCount * 3 + 2] = atof(content);
			}

			else if (m_unPos == M_LIST_VERTS_VERT_U)
			{
				//m_pglModel->getUVData()[m_unListCount].data[0] = atoi(content);
				m_pglModel->getUVData()[m_unListCount * 2] = atoi(content);
			}
			else if (m_unPos == M_LIST_VERTS_VERT_V)
			{
				//m_pglModel->getUVData()[m_unListCount].data[1] = atoi(content);
				m_pglModel->getUVData()[m_unListCount * 2 + 1] = atoi(content);
			}

			else if (m_unPos == M_LIST_FACES_FACE_A)
			{
				//m_pglModel->getFaceData()[m_unListCount].data[0] = atoi(content);
				m_pglModel->getFaceData()[m_unListCount * 3] = atoi(content);
			}
			else if (m_unPos == M_LIST_FACES_FACE_B)
			{
				//m_pglModel->getFaceData()[m_unListCount].data[1] = atoi(content);
				m_pglModel->getFaceData()[m_unListCount * 3 + 1] = atoi(content);
			}
			else if (m_unPos == M_LIST_FACES_FACE_C)
			{
				//m_pglModel->getFaceData()[m_unListCount].data[2] = atoi(content);
				m_pglModel->getFaceData()[m_unListCount * 3 + 2] = atoi(content);
			}
		}

		return S_OK;
	}
};
